<template>
    <v-card>
        <Head title="Categories"/>
        <v-card-title>
            Book Categories
            <v-spacer/>
            <v-btn class="ma-2" color="primary" small @click="newItem">
                Add New
                <v-icon right>mdi-plus</v-icon>
            </v-btn>
        </v-card-title>
        <v-divider/>
        <v-data-table :headers="headers" :items="categories" class="elevation-1">
            <template v-slot:top>
                <v-dialog v-model="dialog" max-width="500px">
                    <v-card>
                        <v-toolbar color="primary" dark>
                            <h3>{{ dialogTitle }}</h3>
                        </v-toolbar>
                        <v-card-text>
                            <v-container>
                                <v-row>
                                    <v-col align="center">
                                        <v-hover>
                                            <template v-slot:default="{ hover }">
                                                <v-avatar size="144px">
                                                    <v-img :src="editedItem.photoPath"/>
                                                    <v-fade-transition>
                                                        <v-overlay v-if="hover || !editedItem.id" absolute color="primary">
                                                            <v-btn v-show="photoChanged" icon @click.stop="removePhoto">
                                                                <v-icon>mdi-delete</v-icon>
                                                            </v-btn>
                                                            <v-btn icon @click.stop="addImage">
                                                                <v-icon>mdi-pencil</v-icon>
                                                            </v-btn>
                                                        </v-overlay>
                                                    </v-fade-transition>
                                                </v-avatar>
                                            </template>
                                        </v-hover>
                                    </v-col>
                                    <v-col cols="12">
                                        <v-text-field v-model="editedItem.name" :rules="rules" autofocus
                                                      label="Category name"></v-text-field>
                                    </v-col>
                                </v-row>
                            </v-container>
                        </v-card-text>
                        <v-divider/>
                        <v-card-actions>
                            <v-btn color="error" depressed text @click="close"> Cancel</v-btn>
                            <v-spacer></v-spacer>
                            <v-btn :loading="loading" color="primary" depressed @click="save">
                                {{ editing ? 'Update' : 'Save' }}
                                <v-icon right small>mdi-content-save</v-icon>
                            </v-btn>
                        </v-card-actions>
                    </v-card>
                </v-dialog>
            </template>
            <template v-slot:item.photo="{ item }">
                <v-avatar class="my-2" tile>
                    <img :alt="item.name" :src="item.photo_path">
                </v-avatar>
            </template>
            <template v-slot:item.name="{ item }">
                {{ item.name }} <i v-if="item.books_count > 0">({{ item.books_count }})</i>
            </template>
            <template v-slot:item.created_at="{ item }">
                {{ item.created_at | dateTime }}
            </template>
            <template v-slot:item.status="{item}">
                <v-chip :color="item.active? 'success':'error'" x-small v-html="item.active ? 'Active' : 'Hidden'"/>
            </template>
            <template v-slot:item.actions="{ item, index }">
                <v-btn class="mr-2" color="success" small @click="editItem(item)">
                    Edit
                    <v-icon right small>mdi-pencil</v-icon>
                </v-btn>
                <v-menu bottom dense left>
                    <template v-slot:activator="{ on, attrs }">
                        <v-btn icon small v-bind="attrs" v-on="on">
                            <v-icon>mdi-dots-vertical</v-icon>
                        </v-btn>
                    </template>
                    <v-list dense>
                        <v-list-item dense link @click="updateStatus(index, item)">
                            <v-list-item-title>
                                <v-icon color="warning" left small>
                                    mdi-{{ item.active ? 'eye-off-outline' : 'eye-outline' }}
                                </v-icon>
                                {{ item.active ? 'Deactivate' : 'Activate' }}
                            </v-list-item-title>
                        </v-list-item>
                        <v-list-item dense link @click="deleteItem(item)">
                            <v-list-item-title>
                                <v-icon color="error" left small>mdi-trash-can-outline</v-icon>
                                Delete
                            </v-list-item-title>
                        </v-list-item>
                    </v-list>
                </v-menu>
            </template>
        </v-data-table>
    </v-card>
</template>


<script>
import DashboardLayout from "../../Components/Layouts/Dashboard.vue";
import CategoryLayout from "./Category.vue";


export default {
    name: "CategoryIndex",
    layout: [DashboardLayout, CategoryLayout],
    data() {
        return {
            error: false,
            rules: [
                value => !!value || 'Please fill this field.',
                value => (value && value.length >= 3) || 'Min 3 characters'
            ],
            dialog: false,
            loading: false,
            headers: [
                {text: '', align: 'center', value: 'photo'},
                {text: 'Name', align: 'start', value: 'name'},
                {text: 'Created on', value: 'created_at'},
                {text: 'Status', value: 'status'},
                {text: 'Created by', value: 'user.name'},
                {text: 'Action', value: 'actions', sortable: false},
            ],
            categories: this.$page.props.categories,
            editedItem: {
                id: null,
                name: '',
                media: null,
                photoPath: ''
            },
            defaultItem: {
                id: null,
                name: '',
                photo: null,
                photoPath: '',
            },
        };
    },
    watch: {
        dialog(val) {
            val || this.close()
        },
    },
    computed: {
        editing() {
            return this.editedItem.id
        },
        dialogTitle() {
            return this.editing ? `Edit category: ${this.editedItem.name}` : "New category"
        },
        photoChanged() {
            if (this.editedItem.id) {
                if (this.editedItem.media) {
                    return this.editedItem.photoPath !== this.editedItem.media.full_path
                }
                return false;
            }

            return false
        },
    },
    methods: {
        removePhoto() {
            if (this.editedItem.media)
                this.editedItem.photoPath = this.editedItem.media.full_path
            else
                this.editedItem.photoPath = ''
        },
        editItem(item) {
            this.editedItem.id = item.id
            this.editedItem.name = item.name
            this.editedItem.photoPath = item.media? item.media.full_path : item.photo_path
            this.dialog = true
        },
        addImage() {
            let eventName = 'cato-imagexx'
            this.eventSubscription = this.$store.subscribe((mutation, state) => {
                if (mutation.type === "Media/SET_EVENT_NAME" && mutation.payload === eventName) {
                    if (state.Media.media.length) {
                        this.editedItem.media = state.Media.media[0]
                        this.editedItem.photoPath = state.Media.media[0].full_path
                        this.eventSubscription()
                    }
                }
            });
            this.showMediaDialog({event: eventName, type: 'image', title: 'Select category image', multiple: false})
        },
        newItem() {
            this.dialog = true
        },
        deleteItem() {
            this._confirmDialog({
                title: "Delete category",
                message: "Are you sure about this?",
                confirm: () => {
                    this.$inertia.delete(this.$route('category.delete', this.editedItem.id), {
                        onBefore: () => {
                        },
                        onSuccess: (page) => {
                            this.showSnack(["Successfully deleted category!"]);
                            this.categories = page.props.categories
                        },
                        onError: (errors) => {
                            this.showSnack([errors, "error"]);
                        },
                        onFinish: visit => {
                        },
                    });
                }
            })
        },
        updateStatus(index, item) {
            this._confirmDialog({
                title: "Change visibility",
                message: item.active ? 'Hidden categories would not be visible on the main site.' : 'This is make this category visible on the main site',
                confirm: () => {
                    axios.post(this.$route('category.status', item.id))
                        .then(resp => resp.data)
                        .then(data => {
                            this.categories[index].active = data.data.active
                        })
                }
            })
        },
        close() {
            this.dialog = false
            this.$nextTick(() => {
                this.editedItem = Object.assign({}, this.defaultItem)
            })
        },
        save() {
            if (this.editing) {
                this.loading = true
                this.$inertia.put(this.$route('category.update', this.editedItem.id), this.editedItem, {
                    only: ['categories'],
                    onBefore: () => {
                    },
                    onSuccess: (page) => {
                        this.showSnack(["Successfully updated category!"])
                        this.categories = page.props.categories
                        this.close()
                    },
                    onError: (errors) => {
                        this.error = true
                        this.showSnack([Object.values(errors)[0], "error"])
                    },
                    onFinish: visit => {
                        this.loading = false
                    },
                });

            } else {
                this.loading = true
                this.$inertia.post(this.$route('category.store'), this.editedItem, {
                    only: ['categories'],
                    onBefore: () => {
                    },
                    onSuccess: (page) => {
                        this.close()
                        this.showSnack(["Category added successfully"])
                        this.categories = page.props.categories
                    },
                    onError: (errors) => {
                        this.error = true
                        this.showSnack([Object.values(errors)[0], "error"])
                    },
                    onFinish: visit => {
                        this.loading = false
                    },
                });
            }
        },
    },
}
</script>

<style scoped>
</style>
